;************************************************
;* Main Program LCD Temperature Readout 	*
;*						*
;* Copyright 2002-2003  Hans OPSTAELE		*
;*   based on code from Wayne Peacock		*
;*						*
;* Displays temperature in .5 C resolution	*
;* on a 1x16LCD panel and controls a plotter 	*
;************************************************
;* See Schematic for Hardware connections	*
;************************************************
 
.Include 	"1200def.inc" 

.def	count1	= r26		
.def	count2	= r27

.def	temp	= r16
.def	data 	= r17

.def	text	= r0
.def	store	= r1

.equ    PIN_MEET = PINB3

       .org $0000 
       
;************************************************
;* Interrupt Vectors				*
;************************************************
       
        rjmp	reset		; Reset Handler
	reti			; IRQ0 Handler
	reti			; Timer/Counter0 Overflow
	reti			; Analog Comparator
	 
;************************************************
;* Main 					*
;************************************************
                   
reset:  

	; Make Control Pins Output
	
	in 	data, DDRB		; PORTB (1 is out )
	ori	data, 0b00000111	; LCD[0=en ; 1= rw ; 2=rs]:=OUT  KEY[3=meet]:=IN 
					; DS1820[4]:=OUT/IN MOTOR_BOUND[5=l,6=c,7=r ]:=IN
	out	DDRB, data
	
	; Make Data pins Outputs
	in 	data, DDRD
	ori	data, 0b01111111	; LCD[0..3] ; MOTOR[4=step ; 5=dir ]
	out	DDRD,data		


	; Power up delay is appox. 16ms	
	rcall	lcdinit			; Set up Port for LCD
	ldi	count1, 0x15		; 2msec
	rcall	delay100		; 21 x 100us
	rcall	DS1820_init		; Set up Port for DS1820
	
	ldi	data, 0x20		; 4 bit mode, 1 lines
	rcall	command
	ldi	data, 0x0C		; Display on, no cursor
	rcall	command

;************************************************
;* Set plotter-arm in start position		*
;************************************************

  	ldi	data, 0x01		; Set on LCD: "CALIB" 
  	rcall	command			; 
	ldi	data, 'c'
	rcall	onechar
	ldi	data, 'a'
	rcall	onechar	
	ldi	data, 'l'
	rcall	onechar	
	ldi	data, 'i'
	rcall	onechar	
	ldi	data, 'b'
	rcall	onechar

	rcall 	stepper_calib

;************************************************
;* Get Temperature from DS1820			*
;************************************************
	
loop:
  	ldi	data, 0x01
  	rcall	command			; Clear LCD
  	
	rcall	DS1820_reset		; Reset DS1820
	bld	temp, WIRE
	sbrc	temp, WIRE
	rjmp	error

	rcall	DS1820_conv		; Do Temp Conversion
	rcall	ds1820_reset
	
	rcall	DS1820_Scr		; Read Data in scratch Pad
		
	; DS18B12 is diff from DS1820  !!! OH
	; resolutie: DS18B12 is x8 (3bits) of DS1820 (thus shift 3x right)
	lsr     tempmsb			; shift right MSB -> C
	ror     templsb			; div 2
	lsr     tempmsb			; shift right MSB -> C
	ror     templsb			; div 2
	lsr     tempmsb			; shift right MSB -> C
	ror     templsb			; C-> div 2
	 
	
;************************************************
;* Display Temperature & plot                   *
;************************************************
		
	rcall	dump_temp
	rcall	stepper_plot

	
;************************************************
;* Wait for next 'meet' signal                  *
;************************************************

wait_keyd:
	sbic	PINB, PIN_MEET 	;'measurement' 
	rjmp	wait_keyd 	; key pushed?
wait_keyu:	
	sbis	PINB, PIN_MEET 	;'measurement' 
	rjmp	wait_keyu 	; key released?

; --------- main loop --------

	rjmp	loop
			


error:	
	ldi	data, 'e'
	rcall	onechar
        ldi	data, 'r'
	rcall	onechar
	ldi	data, 'r'
	rcall	onechar

halt:	rjmp	halt
	
.include "lcd4.asm"
.include "delay.asm"			
.include "DS1820sub1200.asm"
.include "convert.asm"
.include "stepper.asm"

	