;************************************************
;* Stepper motor control routines          	*
;*						*
;* Copyright 2002-2003  Hans OPSTAELE		*
;*   based on code from Wayne Peacock		*
;*						*
;* Displays temperature in .5 C resolution	*
;* on a 1x16LCD panel and controls a plotter 	*
;************************************************
;* See Schematic for Hardware connections	*
;************************************************

; NEEDS...
;.def	templsb		= r2	; -127..128 value of temp (delta=1 per 0.5C !! eg. -24=-12C)
; GLOBAL
;.def   prev_temp	= r28	; prev_temp reading & current pos of plotter-arm.

.def	prev_temp	= r28
.def	delta		= r13

;************************************************
;* Plotter stepper motor 			*
;* 						*
;* Pin Connections				*
;*						*
;* PD4 --> MOT_L1 (see table)			*
;* PD5 --> MOT_L2 (see table)			*
;*						*
;* PB7 --> LBound				*
;* PB6 --> Center Bound (not used)		*
;* PB5 --> RBound				*
;************************************************

;************************************************
;*  MOTOR Leads: 				*
;*  L1 L2 					*
;*  1 	1	Full stop (! do not use)	*
;*  1   0 	Fwd (stijgend)			*
;*  0	1	Reverse	(dalend)		*
;*  0   0	Float (eq. stop)		*
;************************************************


.equ	PIN_LBOUND	= PINB7
.equ	PIN_CBOUND	= PINB5		; not used!
.equ	PIN_RBOUND	= PINB6

.equ	PIN_MOT1	= PIND4
.equ	PIN_MOT2	= PIND5

;************************************************
;* Move right to lbound	 (ie. minimum)		*
;************************************************

stepper_calib:
	cbi	PORTD, PIN_MOT1		; stop motor (CW -> dalend)
	cbi	PORTD, PIN_MOT2		; 
	
	sbis	PINB, PIN_LBOUND 	; 1 = hoog = geen geleiding = geen licht = blokkeer
	sbi	PORTD, PIN_MOT2		; move plotter! DALEND
	
lcalib:
	sbis	PINB, PIN_LBOUND 	; keep moving, until blocked.. 
	rjmp	lcalib	

	cbi	PORTD, PIN_MOT2		; release motor (float)
	ldi	prev_temp, -20		; eq. minimum bound (-10.0C)
	ret

;************************************************
;* Move right to rbound	 (ie. minimum)		*
;************************************************


stepper_plot:
	cbi	PORTD, PIN_MOT1		; move plotter! STIJGEND
	cbi	PORTD, PIN_MOT2	
	
	mov	delta, templsb
	sub 	delta, prev_temp

	brne	stepper_c1		; als <0 : nothing to do!
	ret 				; exit!
	
stepper_c1:	
	tst 	delta	
	brpl	plot_right		; ALS delta < 0 DAN reverse_dir & neg delta
	
	; dalende temperatuur!
	neg 	delta

	; beweeg tot ev. de grens is ber
	sbis	PINB, PIN_LBOUND 	; 1 = hoog = geen geleiding = geen licht = blokkeer
	sbi	PORTD, PIN_MOT2		; DALEND!
	
plot_lwait:
	sbic	PINB, PIN_LBOUND 	; keep moving, until blocked.. 
	rjmp	plot_err

	;wait some for the thingy to move
	ldi	temp, 100		; 480ms = 24 * 20 msec (range: -15..40 = 110 voor moter min2max in 40 sec)
plot_lsw:
	ldi	count1, 200
	rcall	delay100		; 50 x 200 x 100us
	dec	temp
	brne	plot_lsw

	dec	delta
	brne	plot_lwait

	mov	prev_temp, templsb	; current C eq. prev temp
	cbi	PORTD, PIN_MOT2		; DALEND!
	ret
	
	
;	stijdende temperatuur		
plot_right:	
	sbis	PINB, PIN_RBOUND 	; 1 = hoog = geen geleiding = geen licht = blokkeer
	sbi	PORTD, PIN_MOT1		; STIJGEND!
	
plot_rwait:
	sbic	PINB, PIN_RBOUND 	; keep moving, until blocked.. 
	rjmp	plot_err

	;wait some for the thingy to move
	ldi	temp, 100		;range: -15..40 = 110 stappen min2max in 13.50 sec
plot_rsw:				;13.50/110 = 120msec = 6 * 20 ms
	ldi	count1, 200
	rcall	delay100		;  200 x 100us = 20ms
	dec	temp
	brne	plot_rsw
	

	dec	delta
	brne	plot_rwait

	mov	prev_temp, templsb	; current C eq. prev temp
	cbi	PORTD, PIN_MOT1		; STIJGEND!
	ret


plot_err:
	cbi	PORTD, PIN_MOT1		; move plotter! STIJGEND
	cbi	PORTD, PIN_MOT2	
	
	rcall   stepper_calib		; recalibrate...
	ret
